import argparse
import os
import time
import logging
from bs4 import BeautifulSoup, NavigableString
from googletrans import Translator
from tqdm import tqdm

def translate_text(text, translator, target_lang):
    try:
        result = translator.translate(text, dest=target_lang)
        return result.text
    except Exception as e:
        logging.warning(f"Translation error: {e}")
        return text

def count_translatable_nodes(soup):
    return sum(
        1 for node in soup.descendants
        if isinstance(node, NavigableString) and node.strip()
    )

def translate_soup(soup, translator, target_lang, pbar=None):
    def progress_translate_node(node):
        if isinstance(node, NavigableString):
            text = node.strip()
            if text:
                translated = translate_text(text, translator, target_lang)
                time.sleep(0.5)
                if pbar: pbar.update(1)
                return translated
        elif hasattr(node, 'contents'):
            for i, child in enumerate(node.contents):
                if isinstance(child, NavigableString):
                    text = child.strip()
                    if text:
                        translated = translate_text(text, translator, target_lang)
                        time.sleep(0.5)
                        node.contents[i].replace_with(translated)
                        if pbar: pbar.update(1)
                else:
                    progress_translate_node(child)
    progress_translate_node(soup)

def process_file(filepath, output_path, translator, lang):
    logging.info(f"Processing file: {filepath}")
    with open(filepath, 'r', encoding='utf-8') as f:
        soup = BeautifulSoup(f, "html.parser")

    count = count_translatable_nodes(soup)
    logging.info(f"Found {count} translatable texts.")

    with tqdm(total=count, desc="Translating", ncols=80) as pbar:
        translate_soup(soup, translator, lang, pbar)

    os.makedirs(os.path.dirname(output_path), exist_ok=True)
    with open(output_path, 'w', encoding='utf-8') as f:
        f.write(str(soup))
    logging.info(f"✅ Translated saved to: {output_path}")

def process_all_html(input_dir, output_dir, lang):
    logging.info("📂 Scanning directory...")
    translator = Translator()

    total_files = [
        os.path.join(root, file)
        for root, _, files in os.walk(input_dir)
        for file in files if file.endswith(".html")
    ]

    logging.info(f"Found {len(total_files)} file(s).")

    for i, filepath in enumerate(total_files, 1):
        rel_path = os.path.relpath(filepath, input_dir)
        out_path = os.path.join(output_dir, rel_path)
        logging.info(f"\n🔄 ({i}/{len(total_files)}): {rel_path}")
        process_file(filepath, out_path, translator, lang)

def main():
    parser = argparse.ArgumentParser(description="🌐 html2lang - Translate HTML files to any language.")
    parser.add_argument('-l', '--lang', help='Target language code (e.g. uz, fr, en)', required=False)
    parser.add_argument('-h', '--html', help='HTML file name or "all"', required=False)
    parser.add_argument('--creator', action='store_true', help="Show creator info")
    parser.add_argument('--help', action='store_true', help="Show help message")

    args = parser.parse_args()

    if args.creator:
        print("👤 Created by Axmadjon Qaxxorov (@qaxxorov)")
        return

    if args.help or not (args.lang and args.html):
        print("""
📘 Usage:
  html2lang --lang uz --html all
  html2lang -l fr -h index.html
  html2lang --creator
  html2lang --help
        """)
        return

    lang = args.lang
    html_arg = args.html

    INPUT_DIR = "./html"
    OUTPUT_DIR = "./translated_html"

    logging.basicConfig(format="%(asctime)s - %(levelname)s - %(message)s", level=logging.INFO)

    start = time.time()

    if html_arg == "all":
        process_all_html(INPUT_DIR, OUTPUT_DIR, lang)
    else:
        filepath = os.path.join(INPUT_DIR, html_arg)
        outpath = os.path.join(OUTPUT_DIR, html_arg)
        translator = Translator()
        process_file(filepath, outpath, translator, lang)

    duration = round(time.time() - start, 2)
    logging.info(f"🏁 Finished in {duration} seconds.")

