import os
import time
import logging
from bs4 import BeautifulSoup, NavigableString
from googletrans import Translator
from tqdm import tqdm

def translate_text(text, translator, lang):
    try:
        return translator.translate(text, dest=lang).text
    except Exception as e:
        logging.warning(f"Translation error: {e}")
        return text

def count_translatable_nodes(soup):
    return sum(
        1 for node in soup.descendants
        if isinstance(node, NavigableString) and node.strip()
    )

def translate_soup(soup, translator, lang, pbar=None):
    def walk(node):
        if isinstance(node, NavigableString):
            text = node.strip()
            if text:
                translated = translate_text(text, translator, lang)
                time.sleep(0.5)
                if pbar: pbar.update(1)
                return translated
        elif hasattr(node, 'contents'):
            for i, child in enumerate(node.contents):
                if isinstance(child, NavigableString):
                    text = child.strip()
                    if text:
                        translated = translate_text(text, translator, lang)
                        time.sleep(0.5)
                        node.contents[i].replace_with(translated)
                        if pbar: pbar.update(1)
                else:
                    walk(child)
    walk(soup)

def translate_html_file(input_path, output_path, lang):
    logging.info(f"Processing file: {input_path}")
    translator = Translator()

    with open(input_path, 'r', encoding='utf-8') as f:
        soup = BeautifulSoup(f, "html.parser")

    count = count_translatable_nodes(soup)
    logging.info(f"Found {count} translatable texts.")

    with tqdm(total=count, desc="Translating", ncols=80) as pbar:
        translate_soup(soup, translator, lang, pbar)

    os.makedirs(os.path.dirname(output_path), exist_ok=True)
    with open(output_path, 'w', encoding='utf-8') as f:
        f.write(str(soup))

    logging.info(f"✅ Saved translated file to: {output_path}")

def translate_all_html_files(input_dir, output_dir, lang):
    html_files = [
        os.path.join(root, file)
        for root, _, files in os.walk(input_dir)
        for file in files if file.endswith(".html")
    ]

    logging.info(f"Found {len(html_files)} HTML file(s).")

    for i, path in enumerate(html_files, 1):
        rel_path = os.path.relpath(path, input_dir)
        out_path = os.path.join(output_dir, rel_path)
        logging.info(f"\n🔄 ({i}/{len(html_files)}): {rel_path}")
        translate_html_file(path, out_path, lang)
